#!/usr/bin/env python
'''
core_mod.py - core procedures, common to all elements and to the core
program for the FEM. This module is meant to remain 'stable' between
releases.
'''
#
#        (c) Copyright 2008 Algis Kabaila.  All rights reserved.
#   
# This document is free; you can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later version.
#
# This document is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this document; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
# Cambridge, MA 02139, USA.
#
#     Version 0.4.3-x   x = revision number = $Rev: 201 $
#
#     $Id: core_mod.py 201 2009-04-26 02:19:02Z kabalg $
#

import sys
import os

# General functions ------------------------------------------------------

def sgn(x):
    "(x) -> 1 with the sign of x, zero if x is zero."
    if x == 0:
        return 0
    elif x > 0:
        return 1
    else:
        return -1

def printer(fout, line):
    '''Print on sys.stdout as well as in the file "fout". No changes to line.'''
    t = line + '\n'
    print t ,
    fout.write(t)

def lineRead(fIn, fOut):
    '''Read and echo a line. Only return line without "#" character,'''
    while True:
        # read line - loop for ever
        line = fIn.readline()
        # echo line
        fOut.write(line)
        print line,        
        if not ('#' in line):
            break
    return line

def listRead(fIn, fOut):
    ''' Read a line and return a list of strings.'''
    return lineRead(fIn, fOut).split()

def listProcess(fout, lst, frmt):
    '''(fout, lst, frmt) -> a list of numbers and strings.
    lst is a list. frmt specifies integers as "i", floats as "f",
    string as "s". Comment lines with # at the start or anywhere
    else are echoed to the stdio. Notes after data without # are
    allowed and ignored.
    '''
    # line has been split by procedure listRead.
    result = []
    count = 0
    for var in frmt:
        try:
            if var == 'i':
                result.append(int(lst[count]))
            elif var == 'f':
                result.append(float(lst[count]))
            elif var == 's':
                result.append(lst[count])
            else:
                print "Input format error - abandon ship!"
                print 'given format = ', frmt 
                sys.exit([frmt])
        except:
            print 'Error in line: ', lst
            print 'with format  : ', frmt
            print 'Result is    : ', result
            fout.close()
            raise
        count += 1
    return result                          

def abandon(message):
    print '\n' + message
    print 'The program will terminate'
    sys.exit([message])

def getTitle(fin, fout, expectedTitle):
    '''Get sectionTitle, check it against the expectedTitle and return it.'''
    while True:
        line = fin.readline()
        fout.write(line)
        print line,
        if not ('#' in line):
            break
    sectionTitle = line.strip()
    if sectionTitle <> expectedTitle:
        abandon('Expected ' + expectedTitle + ' got ' + sectionTitle)
    return sectionTitle

def checkList(fin, fout, expectedFirstItem):
    '''Read a line, check and remove item 0, return list of items 1..n.'''
    lst = listRead(fin, fout)
    if lst[0] <> expectedFirstItem:
        abandon('This line should start with character ' + expectedFirstItem)
    return lst[1:]

# General functions -  END -----------------------------------------------

def prolog(defaultRootName):
    '''Open input file from a list of files -> alldata list.'''
#    print 'Roots of file names for files in ./dat directory and  with extension .dat.' ;
#    print
    directory = "./dat/" # plugins!
#    directory = "../dat/"
    fileRoots = [v.split('.')[0] for v in os.listdir(directory) if os.path.splitext(v)[1] ==
                 ".dat"]
#    print"List of fileRoots:"
#    print fileRoots
    # Check if there is a CLI parameter for the fileName
    if len(sys.argv) > 1:
        # This part for program run under CLI (Command Line Interface).
        fileRootName = sys.argv[1]
        # Check if the file exists, else just quit.
        fileName = directory + fileRootName + ".dat"
        if not (fileRootName in fileRoots):
            print fileName + ' not found.'
            print 'Program will terminate. Try again!'
            sys.exit(['file not found 001'])
        try:
            fin = file(fileName);
        except(IOError):
            sys.exit(['file not found'])
    else:
        # This part for program run under IDLE.
        # Read the fileRootName from the input and quit if it does not exist.

        fileRootName = raw_input('Enter root of file Name from above list (<CR> selects ' +
                                 defaultRootName + " ) : ")
        if fileRootName == '':
            fileRootName = defaultRootName
        fileName = directory + fileRootName + ".dat"
        if not (fileRootName in fileRoots):
            print fileName + ' not found.'
            print 'Program will terminate. Try again!'
            sys.exit(['file not found 002'])
            sys.exit()
        try:
            fin = file(fileName);
        except(IOError):
            sys.exit(['file not found'])
    outputFileName = fileName.split('/')[-1].split('.')[0] + '.out'
    try:       
        fout = file(outputFileName,'w')
    except(IOError):
        sys.exit(['file not found'])
    print "Output is on the screen and on an output file."
#    print '----------------------------------------------\n'
    return fin, fout

def assemble(Ks, k, locVec):
    "Assemble one element stiffness (k) in to Structure Stiffness (Ks)."
    n = len(locVec)
    for i in range(n):
        for j in range(n):
            x = locVec[i] * locVec[j]
            if x != 0:
                Ks[abs(locVec[i]) - 1, abs(locVec[j]) -1] += sgn(x) * k[i, j]
    return Ks

if __name__ == '__main__':
    print 'Testing of module and compiling core_mod.py'
