#!/usr/bin/env python
'''
core_mod.py - core procedures, common to all elements and to the core
program for the FEM. This module is meant to remain 'stable' between
releases.
'''
#
#        (c) Copyright 2008 Algis Kabaila.  All rights reserved.
#   
# This document is free; you can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later version.
#
# This document is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this document; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
# Cambridge, MA 02139, USA.
#
#     Version 0.4.3-x   x = revision number = $Rev: 201 $
#
#     $Id: core_mod.py 201 2009-04-26 02:19:02Z kabalg $
#

import sys
import os

# General functions ------------------------------------------------------

def sgn(x):
    "(x) -> 1 with the sign of x, zero if x is zero."
    if x == 0:
        return 0
    elif x > 0:
        return 1
    else:
        return -1

def printer(fout, qprint, line):
    '''Print on sys.stdout as well as in the file "fout". No changes to line.'''
    t = line + '\n'
#    print t ,
    qprint(t)
    fout.write(t)

def lineRead(fIn, fOut, qprint):
    '''Read and echo a line. Only return line without "#" character,'''
    while True:
        # read line - loop for ever
        line = fIn.readline()
        # echo line
        fOut.write(line)
        qprint(line)        
        if not ('#' in line):
            break
    return line

def listRead(fIn, fOut, qprint):
    ''' Read a line and return a list of strings.'''
    return lineRead(fIn, fOut, qprint).split()

def listProcess(fout, qprint, lst, frmt):
    '''(fout, lst, frmt) -> a list of numbers and strings.
    lst is a list. frmt specifies integers as "i", floats as "f",
    string as "s". Comment lines with # at the start or anywhere
    else are echoed to the stdio. Notes after data without # are
    allowed and ignored.
    '''
    # line has been split by procedure listRead.
    result = []
    count = 0
    for var in frmt:
        try:
            if var == 'i':
                result.append(int(lst[count]))
            elif var == 'f':
                result.append(float(lst[count]))
            elif var == 's':
                result.append(lst[count])
            else:
                print "Input format error - abandon ship!"
                print 'given format = ', frmt 
                sys.exit([frmt])
        except:
            print 'Error in line: ', lst
            print 'with format  : ', frmt
            print 'Result is    : ', result
            fout.close()
            sys.exit(['error in listProcess'])
        count += 1
    return result                          

def abandon(message):
    print '\n' + message
    print 'The program will terminate'
    sys.exit([message])

def getTitle(fin, fout, qprint, expectedTitle):
    '''Get sectionTitle, check it against the expectedTitle and return it.'''
    while True:
        line = fin.readline()
        fout.write(line)
        qprint(line)
        if not ('#' in line):
            break
    sectionTitle = line.strip()
    if sectionTitle <> expectedTitle:
        abandon('Expected ' + expectedTitle + ' got ' + sectionTitle)
    return sectionTitle

def checkList(fin, fout, qprint, expectedFirstItem):
    '''Read a line, check and remove item 0, return list of items 1..n.'''
    lst = listRead(fin, fout, qprint)
    if lst[0] <> expectedFirstItem:
        abandon('This line should start with character ' + expectedFirstItem)
    return lst[1:]

# General functions -  END -----------------------------------------------

def prolog(filename):
    '''Create the ouput file "fout" to file-write output'''
    fin = open(filename)
    basename = filename.split('.')[0]
    foutname = basename + '.out'
    fout = open(foutname, 'w')   
    return fin, fout

def assemble(Ks, k, locVec):
    "Assemble one element stiffness (k) in to Structure Stiffness (Ks)."
    n = len(locVec)
    for i in range(n):
        for j in range(n):
            x = locVec[i] * locVec[j]
            if x != 0:
                Ks[abs(locVec[i]) - 1, abs(locVec[j]) -1] += sgn(x) * k[i, j]
    return Ks

if __name__ == '__main__':
    print 'Testing of module and compiling core_mod.py'
