#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
fm.py - a program that handles truss members (t) with frame members,
i.e. beams (b) and is structured so as to simplify addition of other
elements.
'''
import sys

#from math import sqrt

from core_mods import matrixM2
from core_mods import core_mod2 as cm_
from core_mods import joint_mod2 as jm_
import mainline2 as main_


# -------------------------------------------------------------------
def st_analysis(filename, qprint):
    '''This is the mainline for Finite Elements, inc. prismatic members.
    fin_name = full name of input file
    line = reader(i)  - reads i-th line of textEdit_1
    printer(line)     - appends a line on textEdit_2
    '''   
    mat_ = matrixM2.Matrix(qprint)
    fin, fout = cm_.prolog(filename)
    glob = main_.Global_variables(fin, fout, qprint, filename)
    modules = glob.modules
    instances = {}
    for key in modules.keys():
        instances[key] = modules[key](mat_) 
    glob.instances = instances
# never ever, never never try to pass glob to any of the instances from plugins!
    material_list = jm_.Material_list(glob)
    material_list.read_material_list()
    glob.material_list_dic = material_list.dic
    section_list  = jm_.Section_list(glob)
    section_list.read_section_list()
    glob.section_list_dic = section_list.dic
    node_list  = jm_.Node_list(glob)
    glob.node_list_dic = node_list.dic
    member_list = jm_.Member_list(glob)
    member_list.read_member_list()
    props = member_list.do_location_vectors()    
    load_cases_list = jm_.Load_cases_list(glob, node_list)
    load_cases_list.read_load_case()
    load_matrix = load_cases_list.do_load_matrix()
    prescribes = jm_.Prescribes(glob, node_list)
    prescribes.read_prescribes()
    cm_.printer(fout, qprint, '\nAll data read in. Solution follows. -->')
    cm_.printer(fout, qprint, 'Load Matrix (transposed):')
    mat_.printm(fout, glob.qprint,  mat_.transposeM(load_matrix))
    qprint ('materials = \n' + material_list.dic.__str__() + '\n')
    qprint ('section   = \n' + section_list.dic.__str__())
    node_list.print_node_list()    
    member_list.print_member_list()
    qprint ('props dictionary.\n')
    main_.print_dictionary(qprint, props)
#    main_.d_print('\nLoad Matrix')
#    main_.d_print(load_matrix)

    structure_stiffness = main_.Structure_stiffness(glob, props,\
                        load_matrix)
    k_structure = structure_stiffness.assemble_stiffness()
    prescribes.modify(k_structure, load_matrix)    
#    
##---------------------------------------------------------------------        
#    qprint('Diagnostic end.')
#    fin.close()
#    fout.close()
#    return
##---------------------------------------------------------------------
#    
    structure_stiffness.solve(mat_)    
    fin.close()
    fout.close()

if __name__ == '__main__':
    global result
    result = ''
    def qprint(line):
        global result
        result += line

    baseName = raw_input('type baseName, viz beam6  ')
    filename = baseName + '.dat'
    st_analysis(filename, qprint)
    print
    print result
 
    
