#!/usr/bin/env python
'''structure.py - GUI for fem2.py with  several textEdit screens:
Data for input, 
Result for analysis result.'''

import sys
import os

from PyQt4.QtCore import (SIGNAL, SLOT, QT_VERSION_STR, PYQT_VERSION_STR,
                          QMetaObject, QRect)
from PyQt4.QtGui  import (QMainWindow, QApplication, QKeySequence, QAction,
    QIcon, QFileDialog, QMessageBox, QWidget, QGridLayout, QTabWidget,
    QFont, QTextEdit, QGridLayout, QMenuBar, QMenu, QStatusBar)

import platform
import helpform

import qrc_structure
import fm

__version__ = "0.1.04"

# v0.1.01 - 2010-10-11 - reduced number of tabs to 2, font courier pitch 10
# v0.1.02 - 2010-11-08 - changed icon for save, renamed gui main program to "structure"
# v0.1.03 - no more Ui_MainWindow
# v0.1.04 - 2010-11-15 - bug chasing - fileSave does NOT work... arrgh...

class MainWindow(QMainWindow):    
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)
        self.setupUi(self)
        status = self.statusBar()
        status.setSizeGripEnabled(False)
        status.showMessage("Ready", 5000)
    
        self.resize(1200, 750)
        self.filename = None
        self.dirty = False
#-------------------------------------------------------------------------------------        
        fileNewAction = self.createAction("&New...", self.fileNew,
                QKeySequence.New, "filenew", "New file name")
        fileOpenAction = self.createAction("&Open...", self.fileOpen,
                QKeySequence.Open, "fileopen",
                "Open an existing file")
        fileSaveAction = self.createAction("&Save", self.fileSave,
                QKeySequence.Save, "filesave", "Save file")
        fileSaveAsAction = self.createAction("Save &As...",
                self.fileSaveAs, icon="filesaveas",
                tip="Save file with a new name")
        filePrintAction = self.createAction("&Print", self.filePrint,
                QKeySequence.Print, "fileprint", "Print the file")
#        action_Quit_Ctrl_Q        
        fileQuitAction = self.createAction("&Quit", self.fileQuit,
                "Ctrl+Q", icon="filequit", tip="Close the application")
#-------------------------------------------------------------------------------------
#        self.menu_File.addAction(fileNewAction)
#        self.menu_File.addAction(fileOpenAction)
#        self.menu_File.addAction(fileSaveAction)
#        self.menu_File.addAction(fileSaveAsAction)
#        self.menu_File.addAction(filePrintAction)
#        self.menu_File.addAction(fileQuitAction)
        self.addActions(self.menu_File, (fileNewAction, fileOpenAction,
                fileSaveAction, fileSaveAsAction, filePrintAction, 
                fileQuitAction))
#-------------------------------------------------------------------------------------
        fileToolBar = self.addToolBar("File")
        fileToolBar.setObjectName("FileToolBar")
        self.addActions(fileToolBar, (fileNewAction, fileOpenAction, fileSaveAction, 
                                      fileSaveAsAction))
#-------------------------------------------------------------------------------------        
        solveStructureAction = self.createAction("So&lve", self.solveStructure,
                "Ctrl+L", "solveStructure", "Solve the structure")
        self.menu_Analysis.addAction(solveStructureAction)
#--------------------------------------------------------------------------------------
        analyseToolbar = self.addToolBar("Analyse")
        analyseToolbar.setObjectName("AnalyseToolBar")
        analyseToolbar.addAction(solveStructureAction)    
#--------------------------------------------------------------------------------------
# Creating actions "long hand" [without use of self.createAction()]
        aboutAction = QAction(QIcon(":/about.png"), "A&bout", self)
        aboutAction.setShortcut("Ctrl+B")
        aboutText = 'About this program'
        aboutAction.setToolTip(aboutText)
        aboutAction.setStatusTip(aboutText)
        aboutAction.triggered.connect(self.about)
#--------------------------------------------------------------------------------------        
        helpAction = QAction(QIcon(":/help.png"), "&Help", self)
        helpAction.setShortcut("Ctrl+H")
        helpText = 'Help about this program'
        helpAction.setToolTip(helpText)
        helpAction.setStatusTip(helpText)#        elif index == 2:
#            self.textEdit = self.textEdit_3

        helpAction.triggered.connect(self.help)
#--------------------------------------------------------------------------------------
# Add Actions "long hand" [without use of self.addActions()]
        self.menu_Help.addAction(aboutAction)
        self.menu_Help.addAction(helpAction)        
        helpToolBar = self.addToolBar("Help")
        helpToolBar.setObjectName("HelpToolBar")
        helpToolBar.addAction(aboutAction)
        helpToolBar.addAction(helpAction)        
#--------------------------------------------------------------------------------------
        self.tabWidget.setCurrentIndex(0)
        self.tabWidget.currentChanged.connect(self.assignTextEdit)
        self.textEdit = self.textEdit_1      
        self.textEdit_1.textChanged.connect(self.setDirty)
#--------------------------------------------------------------------------------------
        quitToolBar = self.addToolBar("Quit")
        quitToolBar.setObjectName("quitToolBar")
        quitToolBar.addAction(fileQuitAction)
#--------------------------------------------------------------------------------------

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QTabWidget(self.centralwidget)
        font = QFont()
        font.setFamily("Courier 10 Pitch")
        font.setPointSize(12)
        self.tabWidget.setFont(font)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_1 = QWidget()
        font = QFont()
        font.setFamily("Courier 10 Pitch")
        font.setPointSize(12)
        self.tab_1.setFont(font)
        self.tab_1.setObjectName("tab_1")
        self.gridLayout_3 = QGridLayout(self.tab_1)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.textEdit_1 = QTextEdit(self.tab_1)
        self.textEdit_1.setObjectName("textEdit_1")
        self.gridLayout_3.addWidget(self.textEdit_1, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_1, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_2 = QGridLayout(self.tab_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.textEdit_2 = QTextEdit(self.tab_2)
        self.textEdit_2.setObjectName("textEdit_2")
        self.gridLayout_2.addWidget(self.textEdit_2, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setGeometry(QRect(0, 0, 800, 21))
        self.menubar.setObjectName("menubar")
        self.menu_File = QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Analysis = QMenu(self.menubar)
        self.menu_Analysis.setObjectName("menu_Analysis")
        self.menu_Help = QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Analysis.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QApplication.translate("MainWindow", "MainWindow", None, QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_1), QApplication.translate("MainWindow", "Data", None, QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), QApplication.translate("MainWindow", "Results", None, QApplication.UnicodeUTF8))
        self.menu_File.setTitle(QApplication.translate("MainWindow", "&File", None, QApplication.UnicodeUTF8))
        self.menu_Analysis.setTitle(QApplication.translate("MainWindow", "&Analysis", None, QApplication.UnicodeUTF8))
        self.menu_Help.setTitle(QApplication.translate("MainWindow", "&Help", None, QApplication.UnicodeUTF8))


    def fileQuit(self):
#        self.textEdit_3.append('fileQuit action')
        self.close()
        
    def setDirty(self):
        '''On change of text in textEdit window, set the flag
        "dirty" to True'''
        index = self.tabWidget.currentIndex()
        if index is not 0:
            return
        if self.dirty:
            return True
        self.dirty = True
        self.updateStatus('self.dirty set to True')

    def clearDirty(self):
        'Clear dirty flag'
        self.dirty = False

    def assignTextEdit(self):
        'set textEdit to the current tag textEdit_x'
        index = self.tabWidget.currentIndex()
        if index == 0:
            self.textEdit = self.textEdit_1
        elif index == 1:
            self.textEdit = self.textEdit_2
        else:
            sys.exit(['invalid tab number'])

        
    def fileNew(self):
        '''Clear textEdit window to start data for a new file.'''
        if not self.okToContinue():
            return
        else:
            self.textEdit_1.setText(' ')
            self.textEdit_2.setText(' ')            
            self.dirty = False
        self.updateStatus('New file selected.')
        
    def okToContinue(self):
        if self.dirty:
            reply = QMessageBox.question(self,
                    "Data Loader - Unsaved Changes",
                    "Save unsaved changes?",
                    QMessageBox.Yes|QMessageBox.No|QMessageBox.Cancel)
            if reply == QMessageBox.Cancel:
                return False
            elif reply == QMessageBox.Yes:
                self.clearDirty()
                return self.fileSave()
        return True

    def okRead(self):
        'Pop-up a warning message.'
        reply = QMessageBox.warning(self,
                "Warning",
                "Opening of files is possible only\n \
                in Data tab!",
                QMessageBox.Ok)
        return True

    def fileOpen(self):
        if self.tabWidget.currentIndex():
            self.okRead()
            return
        if not self.okToContinue():
            return
        dir = (os.path.dirname(self.filename)
               if self.filename is not None else ".")
# The following is from the PyQt Class Reference:
# QString fileName = QFileDialog.getOpenFileName(this, tr("Open File"),
#                "/home", tr("Images (*.png *.xpm *.jpg)"));        
        fname = unicode(QFileDialog.getOpenFileName(self, "Open File",
                                dir, "Data (*.dat *.txt)\nAll Files (*.*)"))
        if fname:
            self.loadFile(fname)
            self.filename = fname
            self.updateStatus('New file opened.')
                
    def loadFile(self, fname=None):
        fl = open(fname)
        text = fl.read()
        self.textEdit.setText(text)
        self.dirty = False

    def fileSave(self):
        '''Save file with current file name.'''
        if self.filename is None:
            return self.fileSaveAs()
        else:
            flname = self.filename
            if flname:
                fl = open(flname, 'w')
                tempText = self.textEdit.toPlainText()
                fl.write(tempText)
                fl.close()
                self.dirty = False
                self.updateStatus('File saved.')
                return True
            else:
                self.updateStatus('Failed to save... ')
                return False
        self.filename = None
        self.dirty = False

    def fileSaveAs(self):
        fname = self.filename if self.filename is not None else "."
        fname = unicode(QFileDialog.getSaveFileName(self,
                "Analysis program - Save File", fname, "Data File (*.*)"))
        if fname:
            if "." not in fname:
                fname += ".dat"
            self.filename = fname
            return self.fileSave()
        return False
        
    def filePrint(self):
        self.textEdit.append("filePrint not implemented")

    def about(self):
        '''Pop up dialog with "about" summary'''
        QMessageBox.about(self, "Structural Analysis",
                """<b>fe_gui.py</b> v {0}
                <p>Copyright &copy; 2010 Algis Kabaila. 
                All rights reserved.
                <p>This program is a prototype of GUI   
                for a Structural Analysis program.
                <p>Python {1} - Qt {2} - PyQt {3} on {4}""".format(
                __version__, platform.python_version(),
                QT_VERSION_STR, PYQT_VERSION_STR,
                platform.system()))

    def help(self):
        '''Start a help system (which resides in the "help" sudirectory'''
        form = helpform.HelpForm("index.html", self)
        form.show()
#--------------------------------------------------------------------------------------    
# Mark Summerfield's shortcuts:
    def addActions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)
    
    def createAction(self, text, slot=None, shortcut=None, icon=None,
                     tip=None, checkable=False, signal="triggered()"):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(":/{0}.png".format(icon)))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            action.triggered.connect(slot)                        
        if checkable:
            action.setCheckable(True)
        return action
#--------------------------------------------------------------------------------------
    def solveStructure(self):
        '''Open a *.dat file and solve the structure there in.'''
#--------------------------------------------------------------------------------- 
        self.textEdit_2.clear()
        fm.st_analysis(self.filename, self.qprintline)
#--------------------------------------------------------------------------------- 

    def qreadline(self, lineNo):
        return unicode(self.textEdit_1.document().\
            findBlockByLineNumber(lineNo).text()).rstrip()

    def qprintline(self, line):
        self.textEdit_2.append(line.rstrip())

    def updateStatus(self, message):
        '''Keep status current.'''
        if self.filename is not None:
            flbase = os.path.basename(self.filename)
            self.setWindowTitle(unicode("Structural Analysis - " + flbase + "[*]") )
            self.statusBar().showMessage(message, 5000)
            self.setWindowModified(self.dirty)
        
if __name__ == "__main__":
    app = QApplication(sys.argv)
    form = MainWindow()
    form.show()
    app.exec_()
    
    
