#!/usr/bin/env python3
'''Matalg.py - module for Matrix ("mat") class.
>>> amat = Matrix(3, 3)
>>> amat[0] = [25., 5., 1.]
>>> amat[1] = [64., 8., 1.]
>>> amat[2] = [144., 12., 1.]
>>> print('amat[2, 0] =', amat[2, 0])
amat[2, 0] = 144.0
>>> print('amat = ')
amat = 
>>> amat.matprint()
[ [25.0, 5.0, 1.0]
  [64.0, 8.0, 1.0]
  [144.0, 12.0, 1.0] ]
>>> row = Matrix(1, 3)
>>> row[0] = [106.8, 177.2, 279.2]
>>> rhs = row.mattranspose()
>>> print('rhs matrix = ')
rhs matrix = 
>>> rhs.matprint()
[ [106.8]
  [177.2]
  [279.2] ]
>>> x = amat.solve(rhs)
>>> # x = (solved with pivoting)
>>> x.matprint()
[ [0.2904761904761916]
  [19.690476190476172]
  [1.0857142857143396] ]
>>> # Solution without pivoting:
>>> # row[0] = [106.8, 177.2, 279.2]
>>> # rhs = row.mattranspose()
>>> # rhs matrix = 
>>> #rhs.matprint()
#[ [106.8]
#  [177.2]
#  [279.2] ]
>>> # x = amat.solve_obsolete(rhs)
>>> # x = 
>>> # x.matprint()
#[ [0.2904761904761916]
#  [19.690476190476172]
#  [1.0857142857143551] ]
>>> # Neater method of input/output:
>>> # Set one term
>>> x[1, 0] = 19.7
>>> x.matprint()
[ [0.2904761904761916]
  [19.7]
  [1.0857142857143396] ]
>>> cmat = Matrix(3, 3)
>>> cmat[0] = [5,  6, 7]
>>> cmat[1] = [10, 12, 3]
>>> cmat[2] = [20,  17, 19]
>>> cmat.matprint()
[ [5, 6, 7]
  [10, 12, 3]
  [20, 17, 19] ]
>>> rhs = Matrix(3, 1)
>>> rhs[0][0] = 18
>>> rhs[1][0] = 25
>>> rhs[2, 0] = 56
>>> x = cmat.solve(rhs)
>>> x.matprint()
[ [1.0]
  [1.0]
  [1.0] ]

'''
# Copyright (c) 2011, Algis Kabaila, All rights reserved.
# This work is licensed under the Creative Commons Attribution 3.0 
# Unported License. To view a copy of this license, visit
# http://creativecommons.org/licenses/by/3.0/ or send a letter to 
# Creative Commons, 171 Second Street, Suite 300, San Francisco, 
# California, 94105, USA. Work AS IS no WARRANTIES!

__version__ = '0.0.6'

# Utility functions. --------------------------------
def find_max(lst):
    '''Find maximum *absolute* value of entries in a list, together
    with its offsetfrom origin.'''
    lst2 = [abs(v) for v in lst]
    index = lst2.index(max(lst2))
    amax = lst[index]
    return amax, index

# Matrix class with its methods.---------------------
class Matrix(list):
    '''"Matrix" creates a (m x n) matrix of float 0.'s'''
    def __init__(self, m=1, n=1):
        '''m = no of rows, n = no of columns.'''
        self.m = m
        self.n = n  
        row = []
        for j in range(n):
            row.append(0.0)
        for i in range(m):
            self.append(row[:])
        self.flag = ' '  # flag signals type of binary operation

    def __getitem__(self, key):
        if type(key) is int:
            return super().__getitem__(key)
        else:    
            if type(key) is tuple:
                if len(key) == 2:
                    i, j = key
                    return self[i][j]
                elif len(key) == 3:
                    i, j, k = key
                    return self[i][j][k]
                elif len(key) == 4:
                    i, j, k, l = key
                    return self[i][j][k][l]       
                else:
                    print('key is not int not or llen(tuple)>4-fail')
                    raise NotImplementedError()

    def __setitem__(self, key, value):
        if type(key) is int:
            super().__setitem__(key, value)
        elif type(key) is tuple:
            if len(key) == 2:
                i, j = key
                self[i][j] = value
            elif len(key) == 3:
                i, j, k = key
                self[i][j][k] = value
            elif len(key) == 4:
                i,j,k,l = key
                self[i][j][k][l] = value                        
            else:
                raise NotImplementedError()
            
    def __str__(self):
        if type(self) != Matrix:
            return NotImplemented
        else:           
            s = '\n'
            for i in range(self.m):
                if i == self.m - 1:
                    s += self[i].__str__()
                else:
                    s += self[i].__str__() + '\n'
            return s    

    def __eq__(self, other):
        if type(self) is Matrix and type(other) is Matrix:
            return self.matequal(other)
        else:
            return NotImplemented
       
    def __add__(self, other):
        if (type(self) is Matrix) and (type(other) is Matrix):
            return self.matadd(other)
        else:
            return NotImplemented

    def __sub__(self, other):
        if (type(self) is Matrix) and (type(other) is Matrix):
            return self.matsub(other)
        else:
            return NotImplemented
        
    def __mul__(self, other):
        '''Matrix multiplication: [ * ] when both self and other
        are matrices. When one is a scalar and the other is a
        matrix, scalar multiplicatin of a matrix.
        '''
        if (type(self) is Matrix) and (type(other) is Matrix):
            assert self.n == other.m, 'Not compatible for multiplication'
##          Matrix multiplication
            return self.matmult(other)
        elif (type(self) is Matrix) and (type(other) in (float, int)):
            self.scalarmult(other)
            return self
        elif (type(self) in (float, int)) and type(other) is Matrix:
            return NotImplemented
        else:
            return NotImplemented
        
    def __rmul__(self, other):
        "number * matrix --> matrix"
        if (type(other) in (float, int)) and type(self) is (Matrix):
            self.scalarmult(other)
            return self

    def __invert__(self):
        '''
        [ ~ ] Matrix inversion).
        '''
        if type(self) is Matrix:
##            self.matinvert()
            return self.matinvert()
        else:
            return NotImplemented

    def __pow__(self, other):
        '''
        [ ** ] Equation solver x = self * rhs ).
        '''
        if type(self) is Matrix and type(other) is Matrix:
            return self.solve(other)
        else:
            return NotImplemented

    def __and__(self, other):
        if type(self) is Matrix and type(other) in (float, int):
            return self.solve(other)
        else:
            return NotImplemented
        
   
    def matprint(self):
        'Usage: to print amat issue amat.matprint() .'
        m = self.m
        n = self.n
        print('[', end=' ') 
        if m < 2:
            print(self[0], ']')
        else:
            for i in range(m):
                if i == 0:
                    print(self[i])                    
                elif i < (m-1):
                    print(' ', self[i])
                else:
                    print(' ', self[i], ']')
                
    def matmult(self, other):
        "self.matmult(other) --> matrix product self x other."
        m = self.m
        n = self.n
        assert self.n == other.m, 'incompatible for mat mult.'
        k = other.n
        result = Matrix(m, k)
        for i in range(m):
            for j in range(k):
                result[i][j] = 0.0
                for l in range(n):
                    result[i][j] += self[i][l] * other[l][j]
        return result
        
    def matadd(self, other):
        'return (mat add) = self + other'
        assert self.m == other.m, 'must be same size'
        assert self.n == other.n, 'must be same size'
        self.flag = 'a'
        return self.matbinop(other)        
        
    def matsub(self, other):
        'return (mat Subtract) = self - other.'
        assert self.m == other.m, 'must be same size'
        assert self.n == other.n, 'must be same size'
        self.flag = 's'
        return  self.matbinop(other)
        
    def scalarmult(self, factor):
        '''Multiply matrix by scalar (in place).
'''
        for i in range(self.m):
            for j in range(self.n):
                self[i][j] = factor * self[i][j]
                
    def matcopy(self):
        'Creates and returns a copy of the matrix.'
        other = Matrix(self.m, self.n)
        for i in range(self.m):
            other[i] = self[i][:]
        return other
        
    def matbinop(self, other):
        'Mat addition and subtraction.'
        assert self.m == other.m, 'must be same size'
        assert self.n == other.n, 'must be same size'
        flag = self.flag
        result = Matrix(other.m, other.n)
        for i in range(other.m):
            for j in range(other.n):
                if flag == 'a': 
                    result[i][j] = self[i][j] + other[i][j]
                elif flag == 's':
                    result[i][j] = self[i][j] - other[i][j]
                else:
                    print('should never happen...')
                    system.exit([1])
        return result
               
    def matunit(self):
        'Make self the unit matrix (in place).'
        m = self.m
        n = self.n
        assert m == n, 'Unary matrix must be square.'
        for i in range(m):
            for j in range(n):
                self[i][j] = 0.0
                if i == j:
                    self[i][j] = 1.0
                
    def mattranspose(self):
        'self.mattranspose --> returns transpose (self unchanged).'
        m = self.m
        n = self.n
        newmat = Matrix(n, m)
        for i in range(m):
            for j in range(n):
                newmat[j][i] = self[i][j]
        return newmat
    
#  --------------------------------------------
# Solution of equations and inverse
    def solve_obsolete(self, rhs):
        '''This is an obsolete method that may be useful
        for debugging; then to be removed.'''
        amat = self
        m = amat.m
        n = rhs.n
        x = Matrix(m, n)
        aug = Matrix(m, m+n) # augmented mat
        for i in range(m):
            for j in range(m):
                aug[i][j] = amat[i][j]
        for i in range(m):
            for j in range((m+n)):
                aug[i][j] = rhs[i][j-m]            
# The naive method - no pivoting, simplest code, solves SOME equations
        for ii in range(m):
            pivot = aug[ii][ii]
            for jj in range(m+n):
                aug[ii][jj] = aug[ii][jj] / pivot
            for i in range(m):
                factor = aug[i][ii]
                for j in range(m+n):
                    if ii != i:
                        aug[i][j] = aug[i][j] - factor * aug[ii][j]
        for i in range(m):
            rhs[i] = aug[i][m:]
        return rhs
        
    def solve(self, other):
        'other is rhs and is returned as solution. Partial pivoting.'
        assert self.m == self.n, 'self must be a square matrix'
        assert self.n == other.m, 'self.n must be equal to other m'
        amat = self
        m = amat.m
        n = other.n
        x = Matrix(m, n)
        aug = Matrix(m, m+n) # augmented mat
        row_index = list(range(m))
        done = []
        for i in range(m):
            for j in range(m):
                aug[i][j] = amat[i][j]
        for i in range(m):
            for j in range(m, (m+n)):
                aug[i][j] = other[i][j-m]  
# Partial Pivoting - somewhat better solver
        for ii in range(m):
            pivcol = []
            rowindeces = []
            for k in range(m):
                if not k in done:
                    pivcol.append(aug[k][ii])
                    rowindeces.append(k) 
            pivot, index = find_max(pivcol)
            cii = rowindeces[index]
            done.append(cii)            
            for jj in range(m+n):
                aug[cii][jj] = aug[cii][jj] / pivot
            for i in range(m):
                factor = aug[i][ii]
                for j in range(m+n):
                    if i != cii:
                        aug[i][j] = aug[i][j] - factor * aug[cii][j]
        for i in range(m):
            other[i] = aug[done[i]][m:]
        return other

    def matinvert(self):
        '''amat.matinvert() --> inverse of amat.
'''       
        other = self.matcopy()
        other.matunit()        
        return self.solve(other)

    def matequal(self, other):
        'Matrices are equal, return True or False.'
        if self.m != other.m or self.n != other.n:
            return False
        else:
            for i in range(self.m):
                for j in range(self.n):
                    try:
                        assert self[i][j] == other[i][j]
                    except AssertionError: 
                        return False
            return True

# -------------------------------- class end ---------------
# global (utility) functions.
##autoprint = True

def mkzeromat(m, n, autoprint=False):
    'm, n --> make (mxn) zero matrix and print by default.'
    mat = Matrix(m, n)
    if autoprint:
        print('Echo check of mkzeromat:')
        mat.matprint()
    return mat

def mkunitmat(m):
    'm --> make (mxm) unit matrix and print by default.'
    mat = Matrix(m, m)
    mat.matunit()
    if autoprint:
        mat.matprint()        
    return mat

def enterdata(m, n, datalist, autoprint=True):
    '''datalist --> create store matrix and enter data into store.
##>>> amat = enterdata(2, 2, [[25., 5.],[64., 8.]])
##Echo check of mkzeromat:
##[ [0.0, 0.0]
##  [0.0, 0.0] ]
##Echo check of enterdata 
##[25.0, 5.0]
##[64.0, 8.0]
##>>> bmat = enterdata(2, 2, [[145, 12], [1, 2]])
##Echo check of mkzeromat:
##[ [0.0, 0.0]
##  [0.0, 0.0] ]
##Echo check of enterdata 
##[145, 12]
##[1, 2]
##>>> res = amat + bmat
##>>> print('res = ', res)
##res =  
##[170.0, 17.0]
##[65.0, 10.0]
##>>> x = res - bmat
##>>> printmat('x =', x)
##x =
##[ [-0.5714285714285714, 0.7142857142857142]
##  [0.42857142857142855, -0.2857142857142857] ]
##
''' 
    assert m == len(datalist),\
            'Data list does not match matrix space.'
    mat = mkzeromat(m, n)
    for i in range(m):
        assert n == len(datalist[i])
        mat[i] = datalist[i]
    if autoprint:
        print('Echo check of enterdata', mat)
    return mat  

def printmat(message, mat):
    'convenience method.'
    print(message)
    mat.matprint()
            
# T.O.D.O:
# 1. Make all methods return a result (no in place ops, viz
# scalarmult
# 2. Chase up bug on amat == bmat returning True (always?)
# 3. Make the autoprint last parameter to all functions (default=True).
# D.O.N.E

if __name__ == '__main__':
    cmat = enterdata(3, 3, [[5,  6, 7],
[10, 12, 3],
[20,  17, 19]])
    rhs = enterdata(3, 1, [[18], [25], [56]])
    sol2 = cmat.solve(rhs)
    rhs.matprint()

##    import doctest
##    doctest.testmod()
##
##    print('amat =')
##    amat = enterdata(2, 3, [[1, 2, 3],
##                            [4, 5, 6]])
##    bmat = enterdata(2, 3, [[1, 2, 3],
##                            [4, 5, 6]])
##    print('bmat = ')
##    bmat.matprint()
##    print('amat = bmat, true or false?', amat == bmat)
##    bmat[1, 1] = 12
##    bmat.matprint()
##    print('amat = bmat, true or false?', amat == bmat)
##    cmat = enterdata(3, 3, [[5,  6, 7],
##[10, 12, 3],
##[20,  17, 19]])
##    rhs = enterdata(3, 1, [[18], [25], [56]])
##    inv = ~ cmat
##    print('Inverse =', inv)
##    solution = inv * rhs
##    print('solution by inversion =',  solution)
##    print('amat after inversion...', cmat)
##    print('rhs after inversion...', rhs)
##    print('repeat the solution by solver.')
##    sol = cmat ** rhs
##    print('Repeated solution =', sol)
##    
    
##    bmat = Matrix(2, 2)
##    bmat[0] = [145., 12.]
##    bmat[1] = [  1,   2]
##    print('bmat = ')
##    bmat.matprint()
##    res = amat + bmat
####    res = amat.matadd(bmat)
##    print('res = ')
##    res.matprint()
##    x = res - bmat
##    print('x = res - bmat =')
##    x.matprint()
##    print('x == amat ?')
##    print(x == amat)
##    print('type(x) =', type(x) )
##    x[1, 0] = 65
##    x.matprint()
##    print('x == amat ?')
##    print(x == amat)
### This should return False - bug #1    
##    amat = Matrix(2, 2)
##    amat[0] = [2., 5.]
##    amat[1] = [3., 4.]
##    print('amat = ')
##    amat.matprint()
##    bmat = Matrix(2, 2)
##    bmat[0] = [3., 4.]
##    bmat[1] = [  1,   2]
##    print('bmat = ')
##    bmat.matprint()
##    res = amat * bmat
##    print('Matrix product amat * bmat')
##    res.matprint()
##    res = amat * 1.5
##    print('Scalar multiplication of amat * 1.5')
##    res.matprint()
##    res = 2 * amat
##    print('Scalar multiplication 2 * amat =')
##    res.matprint()
### scalarmult has a bug - see T.O.D.O   1. in place scaling is nbg idea.   
##    amat = Matrix(2, 2)
##    amat[0] = [2., 5.]
##    amat[1] = [3., 4.]
##    print('amat = ')
##    amat.matprint()
##    print('inverse of amat =')
##    inv = amat ** -1
##    inv.matprint()
##    print('check the inverse by matmult:')
##    unit = amat * inv
##    unit.matprint()
##    unit2 = inv * amat
##    unit2.matprint()
