#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
fm.py - a program that handles truss members (t) with frame members,
i.e. beams (b) and is structured so as to simplify addition of other
elements.
'''
#
#        (c) Copyright 2011 Algis Kabaila.  All rights reserved.
#   
# This document is free; you can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later version.
#
# This document is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this document; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import io

from core_mods import Matalg as _m

from core_mods import core_mod3 as cm_
from core_mods import joint_mod3 as jm_
import mainline3 as main_

# -------------------------------------------------------------------
class Analysis(object):
    
    def __init__(self, fin, filename, printline):
        self.fin = fin
        self.filename = filename
        self.printline = printline
        
    def st_analysis(self):
        '''This is the mainline for Finite Elements, inc. prismatic members.
        fin_name = full name of input file
        line = reader(i)  - reads i-th line of textEdit_1
        printer(line)     - appends a line on textEdit_2
        '''   
        fin = self.fin
        filename = self.filename
        printline = self.printline
        fdummy, fout = cm_.prolog(filename)
    #   fin is defined as fin = io.StringIO(dataBall)
        glob = main_.Global_variables(fin, fout, printline, filename)
        modules = glob.modules
        instances = {}
        for key in list(modules.keys()):
            instances[key] = modules[key](_m) 
        glob.instances = instances
    # never ever, never never try to pass glob to any of the instances from plugins!
        material_list = jm_.Material_list(glob)
        material_list.read_material_list()
        glob.material_list_dic = material_list.dic
        section_list  = jm_.Section_list(glob)
        section_list.read_section_list()
        glob.section_list_dic = section_list.dic
        node_list  = jm_.Node_list(glob)
        glob.node_list_dic = node_list.dic    
        member_list = jm_.Member_list(glob)
        member_list.read_member_list()
        props = member_list.do_location_vectors()    
        load_cases_list = jm_.Load_cases_list(glob, node_list)
        load_cases_list.read_load_case()
        load_matrix = load_cases_list.do_load_matrix()
        prescribes = jm_.Prescribes(glob, node_list)
        prescribes.read_prescribes()
        cm_.printer(fout, printline, '\nAll data read in. Solution follows. -->')
        cm_.printer(fout, printline, 'Load Matrix (transposed):')
        print(load_matrix.mattranspose())
        printline ('materials = \n' + material_list.dic.__str__() + '\n')
        printline ('section   = \n' + section_list.dic.__str__())
        node_list.print_node_list()    
        member_list.print_member_list()
        printline ('props dictionary.\n')
        main_.print_dictionary(printline, props)
        structure_stiffness = main_.Structure_stiffness(glob, props,\
                            load_matrix)
        k_structure = structure_stiffness.assemble_stiffness()
        prescribes.modify(k_structure, load_matrix)   
        structure_stiffness.solve(_m)
        fin.close()
        fout.close()

def lineread(fin, printline):
    '''Read and echo a line. Only return line without "#" character,'''
    while True:
        line = fin.readline()
        printline(line)       
        if not ('#' in line):
            break
    return line       

def main(printline, name, dataBall, NameIsMain=False):
    printline('Echo check of data follows:')
    fin = io.StringIO(dataBall)
    for line in fin:
        if NameIsMain:
            print(line.strip())
        else:
            printline(line.strip())
    fin.seek(0, 0)  # rewind file
    printline('*** Data Echo Finished ***\n')
    printline(' ')
    printline('name = ' + name)
    analysis = Analysis(fin, name, printline)
    analysis.st_analysis()
    fin.close()
if __name__ == '__main__':
    
    def printline(line):
        print(line, end='')
    print('Usage:')
    print(' fm.py [name of example] in the "dat" subdirectory;')
    print('Default is "truss1.dat"')
    if len(sys.argv) > 1:
        name = sys.argv[1]
        print('d* name = ' + name)
    else:
        name = 'truss1.dat'
    try:
        finput = open('dat/' + name, 'r')
        dataBall = finput.read()
        finput.close()
    except IOError:
        print('Failed to open given file name.')
        sys.exit(1)
    main(printline, name, dataBall, NameIsMain=True)
 
    
