''' beam_mod.py - module.'''

from math import sqrt

class Beam:
    '''beam element properties for static data and for element properties, 
    bmat, kbar, k, Stress matrix'''
    dic = {'element_type' : 'Beam', 'tag' : 'b',  'material_items' : 2, 
    'section_items' : 2, 'coord_no' : 2, 'freedom_no' : 3, 'no_of_nodes' : 2,
    'nodal_load_no' : 3} 
# Coded, but not checked at all.  ak: 2009-11-19
    def __init__(self, mat_):
        self.mat_ = mat_

    def find_element_properties(self, line, node_list_dic,\
                                 material_list_dic, section_list_dic):
        '''line = member_input_line = member no, tag, end1 no, end2 no, 
        section no, material no. Expected to return 
        (tag, end1-X,Y end2-X,Y, area, i_x, E, location_vector). '''
        tag = line[0]
        if tag is not 'b':
            raise Truss_exception, 'Can only handle truss member data.'
#        line = ['b', 2, 3, 1, 1]
#        line = member_input_line
        node1 = node_list_dic[line[1]]
        node2 = node_list_dic[line[2]]
        d_x = node2[3] - node1[3]
        d_y = node2[4] - node1[4]
        loc_vect = node1[5 : 8] + node2[5 : 8]
        material_no = line[3]
        section_no = line[4]
        e_mod = material_list_dic[material_no]
        area  = section_list_dic[section_no][0]
        i_x   = section_list_dic[section_no][1]
        return d_x, d_y, e_mod, area, i_x, loc_vect
    
    def bmat_kbar(self, d_x, d_y, e_mod, area, i_x):
        '''
        Get frame member kbar stiffness and B matrix and member length length.
        '''   
        mat_ = self.mat_
        b_mat = mat_.zerosM(6, 3)   # b_mat for frames is (6x3)
        length = sqrt(d_x * d_x + d_y * d_y)
        cosx = d_x/length  # cos alpha
        cosy = d_y/length  # cos beta
        b_mat[0, 0] =  -cosx
        b_mat[0, 1] =   cosy
        b_mat[1, 0] =  -cosy
        b_mat[1, 1] =  -cosx    
        b_mat[2, 1] =  -length
        b_mat[2, 2] =  -1
        # ----------
        b_mat[3, 0] =   cosx
        b_mat[3, 1] =  -cosy
        b_mat[4, 0] =   cosy
        b_mat[4, 1] =   cosx
        b_mat[5, 2] =   1
        kbar = mat_.zerosM(3, 3)
        kbar[0, 0] = area / i_x
        kbar[1, 1] = 12./(length * length)
        kbar[1, 2] = -6. / length
        kbar[2, 1] = -6. / length
        kbar[2, 2] =  4
        kbar = (e_mod * i_x / length) * kbar
        return length, b_mat, kbar    
        
#-----------------------------------------------

    def k_element(self, d_x, d_y, e_mod, area, i_x):  #getk(b_mat, kbar):
        '''Get the beam stiffness matrix.'''
        mat_ = self.mat_
        length, b_mat, kbar = self.bmat_kbar(d_x, d_y, e_mod, area, i_x)
        b_transposed = mat_.transposeM(b_mat)
        temp = mat_.multM(b_mat, kbar)
        return mat_.multM(temp, b_transposed) 
    
    def stress_matrix(self, d_x, d_y, e_mod, area, i_x): # (length, b_mat, kbar):
        'Find Stress Matrix for a beam (frame member).'
        mat_ = self.mat_    
        length, b_mat, kbar = self.bmat_kbar(d_x, d_y, e_mod, area, i_x)
        a_mat = mat_.zerosM(4, 3)
        a_mat[0, 0] = a_mat[1, 1] = a_mat[2, 2] = a_mat[3, 2] = 1
        a_mat[2, 1] = length
        b_transposed = mat_.transposeM(b_mat)            
        temp = mat_.multM(kbar, b_transposed)
        stress_matrix = mat_.multM(a_mat, temp)
        return stress_matrix

if __name__ == '__main__':
    print
    var = Beam
    keys = Beam.dic.keys()
    print 'dictionary "dic" keys ='
    print keys
    print 'Dictionary "dic" ='
    print Beam.dic
