#!/usr/bin/env python
''' truss_mod.py - module.'''

from math import sqrt

class Truss_exception(Exception):
    pass

class Truss:
    '''truss element properties for static data and for element properties, 
    bmat, kbar, k, Stress matrix'''
    dic = {'element_type' : 'Truss', 'tag' : 't', 'material_items' : 1,
    'section_items' : 1, 'coord_no' : 2, 'freedom_no' : 2,
    'no_of_nodes' : 2, 'nodal_load_no' : 2} 
    def __init__(self, mat_):
        self.mat_ = mat_
#        self.switch = switch
#        # If switch is not None, return data for plotting and exit.
        
    def find_element_properties(self, line, node_list_dic,\
                                 material_list_dic, section_list_dic):
        '''line = member_input_line = member no, tag, end1 no, end2 no, 
        section no, material no. Expected to return 
        (tag, end1 X,Y end2 X,Y, section, E, location_vector). '''
        tag = line[0]
        if tag is not 't':
            raise Truss_exception, 'Can only handle truss member data.'
#        line = ['t', 2, 3, 1, 1]
#        line = member_input_line
        node1 = node_list_dic[line[1]]
        node2 = node_list_dic[line[2]]
        d_x = node2[3] - node1[3]
        d_y = node2[4] - node1[4]
        loc_vect = node1[5:7] + node2[5:7]
        material_no = line[3]
        section_no = line[4]
        e_mod = material_list_dic[material_no]
        area  = section_list_dic[section_no]
        return d_x, d_y, e_mod, area, loc_vect
        
    def bmat_kbar(self, d_x, d_y, e_mod, area):
        "Compute matrix B and member stiffness kbar for truss bar."
        mat_ = self.mat_
        b_mat = mat_.zerosM(4, 1)
        length = sqrt(d_x * d_x + d_y * d_y)
        cosx = d_x/length  # cosx
        cosy = d_y/length  # sin
        b_mat[0, 0] = -cosx
        b_mat[1, 0] = -cosy
        b_mat[2, 0] =  cosx
        b_mat[3, 0] =  cosy
        kbar = (e_mod * area) / length
        return b_mat, kbar    

    def k_element(self, d_x, d_y, e_mod, area):
        'Compute k_mat for truss bar.'
        mat_ = self.mat_
        b_mat, kbar = self.bmat_kbar(d_x, d_y, e_mod, area)
        b_transposed = mat_.transposeM(b_mat)
        k_mat = kbar * mat_.multM(b_mat, b_transposed)
        return  k_mat

    def stress_matrix(self, d_x, d_y, e_mod, area):
        '''Returns truss truss bar's stress matrix.'''
        mat_ = self.mat_
        b_mat, kbar = self.bmat_kbar(d_x, d_y, e_mod, area)
        b_transposed = mat_.transposeM(b_mat)
        # kbar is a scalar for truss member
        stress_mat = kbar * b_transposed
        return stress_mat
# end class Truss:

if __name__ == '__main__':
# ---------------------------------------------
    ok = True
    try:
        import matrixM
    except ImportError:
        ok = False
# for the import to work, the matrixM module needs to be on Python path.
    print
    keys = Truss.dic.keys()
    print 'Dictionary "Truss.dic" ='
    print Truss.dic
    print 'Is import of matrixM successful? Ans: ', ok
# The following is only attempted if the matrixM is imported and 
# numpy is installed.
    if ok:
        print 'It is OK, ', ok
        mat_ = matrixM.Matrix('numpy')
        truss = Truss(mat_)
# The following sample data is taken from program output for truss1.dat
# Format node_list_doc[i] = ['t', 'm', 'no', x, y, freedomX, freedomY]                            
        node_list_dic = {1: ['t', 'm', '1', 0.0, 0.0, 0, 0],
                         2: ['t', 'm', '2', 4000.0, 0.0, 1, 2],
                         3: ['t', 'm', '3', 8000.0, 0.0, 3, 0],
                         4: ['t', 'm', '4', 4000.0, 3000.0, 4, 5]}
# Format member_list_dic[i] = 't', end1, end2, section no, m
        member_list_dic = {1: ['t', 1, 2, 1, 1],
                           2: ['t', 2, 3, 1, 1],
                           3: ['t', 2, 4, 1, 1],
                           4: ['t', 1, 4, 1, 1],
                           5: ['t', 4, 3, 1, 1]}
        material_list_dic =  {1: 200000.0}
        section_list_dic =   {1: 1600.0}
# Take member no 2, for example
        line = ['t', 2, 3, 1, 1]
        properties = truss.find_element_properties(line, node_list_dic, material_list_dic, section_list_dic)
        print 'properties = (d_x, d_y, e_mod, area [loc_vect]) = \n', properties
# Stiffness matrix
        d_x, d_y, e_mod, area,  loc_vect = properties
        k_el = truss.k_element(d_x, d_y, e_mod, area)
        print 'element stiffness matrix, k_el = \n'
        mat_.printM(k_el)
#  Stress matrix
        stress_mat = truss.stress_matrix(d_x, d_y, e_mod, area)
        print 'stress matrix '
        mat_.printM(stress_mat)
